﻿EnableExplicit

;- -- Information.
; Dieses Module ist die Hauptdatei des Projekts.

;- -- Version 01.
; Status:   Fertig.
; Autor:    Peter Dreisiebner
; E-Mail:   web@dreisiebner.at
; Website:  http://peter.dreisiebner.at
; Geändert: 2017-10-12 - PureBasic v5.61 (x86).
;
; 2017-10-12 Version 01:
;  DebugView-v02.pbi mit der Unterstützung von Threads eingebunden.

;-
;- -- Include.
;
XIncludeFile "Common.pbi"        ; Datei mit den Debug-Konstanten und der Datei CommonDebug.pbi einbinden.
XIncludeFile "DebugView-v02.pbi" ; Das DebugView-Modul einbinden.

UseModule Common

;-
;- -- DeclareModule Application.
;
DeclareModule Application
   EnableExplicit

   Declare Main()

EndDeclareModule  ; Application.

;-
;- Module Application.
;
Module Application
   EnableExplicit
   UseModule Common
   DebugOutApplication  ; Macro mit den Debug-Konstanten aus der Datei CommonDebug.pbi.
   
   ;-
   ;- -- Variablen.
   Global miThreadCounter.i            ; Zähler für die Anzahl der aktuellen Threads.
   Global miThreadCounterMax.i         ; Zähler für die Gesamtanzahl der verwendeten Threads.
   Threaded miThreadedThreadCounter.i  ; Der Zähler wird nur innerhalb der Threads verändert.
   Global miMutex.i
   
   ;-
   
   Procedure.i Test3(iInstanceNumber.i)
      DebugView::ProcIn(iInstanceNumber, miThreadedThreadCounter) ; Macro für den Prozedurbeginn mit optionalen Parametern.
      
      ; leer.
      
      DebugView::ProcOut(iInstanceNumber, miThreadedThreadCounter)   ; Macro für das Prozedurende mit optionalen Parametern.
   EndProcedure
   
   
   Procedure.i Test2(iInstanceNumber.i)
      DebugView::ProcIn(iInstanceNumber, miThreadedThreadCounter)
      
      Test3(iInstanceNumber)
      
      DebugView::ProcOut(iInstanceNumber, miThreadedThreadCounter)
   EndProcedure
   
   
   Procedure.i Test1(iInstanceNumber.i)
      DebugView::ProcIn(iInstanceNumber, miThreadedThreadCounter)
      
      If miThreadedThreadCounter
         DebugView::Temp("Die Threadnummer steht in den geschweiften Klammern.", iInstanceNumber, miThreadedThreadCounter)
      EndIf
      
      Test2(iInstanceNumber)
      
      DebugView::ProcOut(iInstanceNumber, miThreadedThreadCounter)
   EndProcedure
   
   
   Procedure.i Thread(iInstanceNumber.i)
      LockMutex(miMutex)
      miThreadCounter + 1     ; Zähler für die Anzahl der aktuellen Threads.
      miThreadCounterMax + 1  ; Zähler für die Gesamtanzahl der verwendeten Threads.
      miThreadedThreadCounter = miThreadCounterMax ; Die Gesamtanzahl ist auch die Nummer des Threads für die Debugausgaben.
      UnlockMutex(miMutex)
      DebugView::ThreadIn(iInstanceNumber, miThreadedThreadCounter)  ; Macro für den Threadbeginn mit optionalen Parametern.
      
      ; Aufruf einiger Prozeduren.
      Test1(iInstanceNumber)
      
      ; Thread beenden.
      LockMutex(miMutex)
      miThreadCounter - 1  ; Zähler für die Anzahl der aktuellen Threads.
      DebugView::Info("ThreadCounter = " + miThreadCounter, iInstanceNumber, miThreadedThreadCounter)
      UnlockMutex(miMutex)
      DebugView::ThreadOut(iInstanceNumber, miThreadedThreadCounter) ; Macro für das Threadende mit optionalen Parametern.
   EndProcedure
   
   
   Procedure Main()
      ; Startprozedur des Programms.
      DebugView::Start("Programm >" + #sApplicationName + "< gestartet.")  ; Macro für den Programmstart mit optionalen Parametern.
      DebugView::ProcIn()  ; Macro für den Prozedurbeginn mit optionalen Parametern.
      Protected iInstanceNumber.i, iThreadNumber1.i, iThreadNumber2.i
      
      iInstanceNumber = 1  ; Beispiel für eine Instanznummer z.B. eines Fensters.
      
      DebugView::Info("Ausgabe einer Information.")
      DebugView::Info("Ausgabe einer Information mit Instanznummer in eckigen Klammern.", iInstanceNumber)
      DebugView::Successful("Ausgabe für Erfolg.")
      DebugView::Error("Ausgabe eines Fehlers.")
      DebugView::Temp("Temporäre Ausgabe.")
      
      ; Aufruf einiger Prozeduren.
      Test1(iInstanceNumber)
      
      ; Aufruf einer Prozedur aus einem Thread.
      miMutex = CreateMutex()
      iThreadNumber1 = CreateThread(@Thread(), iInstanceNumber)
      iThreadNumber2 = CreateThread(@Thread(), iInstanceNumber)
      
      ; Aufruf einiger Prozeduren.
      Test1(iInstanceNumber)
      
      ; Geschwindigkeitsmessung.
      DebugView::SpeedStart()
      Delay(10)
      DebugView::SpeedStop("Zeitmessung")
      
      FreeMutex(miMutex)
      
      DebugView::ProcOut() ; Macro für das Prozedurende mit optionalen Parametern.
      DebugView::Exit("Programm >" + #sApplicationName + "< beendet.")  ; Macro für das Programmende mit optionalen Parametern.
   EndProcedure


EndModule   ; Application.


;-
;- -- Main
;
; Hinweis für die Debugversion ausgeben.
CompilerIf #fDebugOut = #True
   MessageRequester(#sApplicationName, "Debugversion für das System (DebugView).")
CompilerEndIf

;- Programm starten.
End Application::Main()
