﻿EnableExplicit

;- -- Information.
; Dieses Modul wird für die Debugausgaben an das Betriebssystem verwendet. Für die Ansicht der Ausgaben wird das
; Programm DebugView v4.81 von Mark Russinovich verwendet. Die Ausgaben können mit dem Programm eingefärbt und gefiltert werden.
;
; Dieses Modul sollte in jedes Projekt mit >XIncludeFile "..\..\_ PureBasic\DebugView-v02.pbi"<
; in die Hauptdatei, z.B. Application.pb eingebunden werden.
;
; Aktiviert wird die Debugausgabe mit den Konstanten in den Modulen Common.pbi und CommonDebug.pbi sowie den jeweiligen
; Macros DebugOut* bzw. Modulkonstanten #DebugOutModule, #DebugSpeedModule und #DebugErrorModule.

;- -- Version 02.
; Status:   Fertig.
; Autor:    Peter Dreisiebner
; Website:  http://peter.dreisiebner.at
; E-Mail:   web@dreisiebner.at
; Geändert: 2017-10-12 - PureBasic v5.61 (x86).

;-
;- -- DeclareModule DebugView.
;
DeclareModule DebugView
   EnableExplicit
   
   ;-
   ;- -- Macros.
   Macro Clear(Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut
         DebugView::Out(#Null$, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eClear, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro Error(Text = #Empty$, Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut Or #fDebugError And #DebugErrorModule
         DebugView::Out(Text, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eError, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro Exit(Text = #Empty$, Instance = #Null)
      CompilerIf #fDebugOut
         DebugView::Out(Text, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eExit, Instance)
      CompilerEndIf
   EndMacro

   Macro Info(Text = #Empty$, Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut Or #fDebugInfo And #DebugOutModule
         DebugView::Out(Text, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eInfo, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro ProcIn(Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut And #fDebugHigh And #DebugOutModule
         DebugView::Out(#Null$, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eProcIn, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro ProcOut(Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut And #fDebugHigh And #DebugOutModule
         DebugView::Out(#Null$, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eProcOut, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro SpeedStart()
      CompilerIf #fDebugOut Or #fDebugSpeed And #DebugSpeedModule
         CompilerIf Not Defined(iDebugSpeedStart, #PB_Variable)
            Protected iDebugSpeedStart.i
         CompilerEndIf
         iDebugSpeedStart = ElapsedMilliseconds()
      CompilerEndIf
   EndMacro

   Macro SpeedStop(Text = #Empty$, Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut Or #fDebugSpeed And #DebugSpeedModule
         CompilerIf Not Defined(iDebugSpeedStop, #PB_Variable)
            Protected iDebugSpeedStop.i
         CompilerEndIf
         iDebugSpeedStop = (ElapsedMilliseconds() - iDebugSpeedStart)
         DebugView::Out(Text + " " + iDebugSpeedStop + " ms",
                        #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eSpeed, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro Start(Text = #Empty$, Instance = #Null)
      CompilerIf #fDebugOut
         DebugView::Out(Text, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eStart, Instance)
      CompilerEndIf
   EndMacro

   Macro Successful(Text = #Empty$, Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut Or #fDebugSuccess And #DebugOutModule
         DebugView::Out(Text, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eSuccessful, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro Temp(Text = #Empty$, Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut Or #fDebugTemp And #DebugOutModule
         DebugView::Out(Text, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eTemp, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro ThreadIn(Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut And #fDebugHigh And #DebugOutModule
         DebugView::Out(#Null$, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eThreadIn, Instance, Thread)
      CompilerEndIf
   EndMacro

   Macro ThreadOut(Instance = #Null, Thread = #Null)
      CompilerIf #fDebugOut And #fDebugHigh And #DebugOutModule
         DebugView::Out(#Null$, #PB_Compiler_Module, #PB_Compiler_Procedure, DebugView::#eThreadOut, Instance, Thread)
      CompilerEndIf
   EndMacro
   
   ;-
   ;- -- Enumeration.
   ;- eEvent.
   Enumeration eEvent
      #eClear
      #eError
      #eExit
      #eInfo
      #eProcIn
      #eProcOut
      #eSpeed
      #eStart
      #eSuccessful
      #eTemp
      #eThreadIn
      #eThreadOut
   EndEnumeration
   
   ;-
   ;- -- Declares.
   Declare Out(sText.s = #Null$, sModule.s = #Null$, sProcedure.s = #Null$, eEvent.i = #eTemp,
               iInstance.i = #Null, iThread.i = #Null)

EndDeclareModule  ; DebugView.

;-
;- -- Module DebugView.
;
Module DebugView
   EnableExplicit
   UseModule Common
   
   ;-
   ;- -- Konstanten.
   #iIndentionWidth = 2
   #sIndentionSign = ". . . . . . . . . . . . . . . . . . . . " ; 40 Zeichen, wie #iMaxIndention.
   #iMaxIndention = 40
   #sMaxIndentionSign = "~"
   #sCounter = "00000"
   #iCounterWidth = 5
   #sDebugViewClear = "DBGVIEWCLEAR"
   
   ;-
   ;- -- Variablen.
   Global miCounterRow.i, miCounterIndention.i
   Global miMutex.i  ; Mutex für die Verwenung in Threads.

   ;-
   
   Procedure Out(sText.s = #Null$, sModule.s = #Null$, sProcedure.s = #Null$, eEvent.i = #eTemp,
                 iInstance.i = #Null, iThread.i = #Null)

      Protected sDebugOut.s, sProcInOut.s, sInstance.s, sProc.s
      Protected iCounterIndention.i
      Protected fDebugViewClear.i = #False
      
      ; Mutex erstellen und verwenden.
      If (Not miMutex)
         miMutex = CreateMutex()
         If (Not miMutex)
            OutputDebugString_(Str(miCounterRow) + ":Err: DebugView::Out: Der Mutex konnte nicht erstellt werden.")
         EndIf
      EndIf
      If (miMutex)
         LockMutex(miMutex)
      EndIf

      ; Zähler für das Einrücken der Ausgaben merken.
      iCounterIndention = miCounterIndention

      ; Die Ausgaben für die verschiedenen Ereignisse wie "In: ", "Out: " usw. werden im Programm "DebugView"
      ; zur Filterung bzw. Farbhervorhebung benutzt.
      Select eEvent

         Case #eClear
            sDebugOut = #sDebugViewClear
            fDebugViewClear = #True

         Case #eError
            sDebugOut = "Err: "

         Case #eExit
            sDebugOut = "End: "

         Case #eInfo
            sDebugOut = "Inf: "

         Case #eProcIn
            If (Not iThread)
               sDebugOut = "In : "
               sProcInOut = ">"
               ; Nachfolgende Debug-Ausgaben einrücken.
               miCounterIndention = miCounterIndention + #iIndentionWidth
            Else
               ; Bei Prozeduren die in Threads laufen, statt 'ProcIn' die Werte für ThreadIn verwenden.
               ; Threads werden ja unterbrochen, und somit stimmt die Einrückung der Debug-Ausgabe nicht mehr.
               sDebugOut = "ThI: "
               sProcInOut = ">"
            EndIf
            
         Case #eProcOut
            If (Not iThread)
               sDebugOut = "Out: "
               sProcInOut = "<"
               ; Einrückung verringern, die Prozedur wird verlassen.
               miCounterIndention = miCounterIndention - #iIndentionWidth
               If (miCounterIndention < #Null)
                  miCounterIndention = #Null
               EndIf
               iCounterIndention = miCounterIndention
            Else
               ; Bei Prozeduren die in Threads laufen, statt 'ProcOut' die Werte für ThreadOut verwenden.
               ; Threads werden ja unterbrochen, und somit stimmt die Einrückung der Debug-Ausgabe nicht mehr.
               sDebugOut = "ThO: "
               sProcInOut = "<"
            EndIf

         Case #eSpeed
            sDebugOut = "Spd: "

         Case #eStart
            sDebugOut = "Str: "

         Case #eSuccessful
            sDebugOut = "Scs: "

         Case #eTemp
            sDebugOut = "Tmp: "

         Case #eThreadIn
            ; Bei Threads werden statt 'ProcIn' und 'ProcOut' die Werte 'ThreadIn' und 'ThreadOut' verwendet.
            ; Threads werden ja unterbrochen, und somit stimmt die Einrückung der Debug-Ausgabe nicht mehr.
            sDebugOut = "ThS: "
            sProcInOut = ">"
            If (Not Len(sText))
               sText = "Thread gestartet."
            EndIf

         Case #eThreadOut
            sDebugOut = "ThE: "
            sProcInOut = "<"
            If (Not Len(sText))
               sText = "Thread beendet."
            EndIf

         Default
            sDebugOut = "Err: "
            sText = "Interner Fehler! - Aufruf von DebugView::Out(...) mit unbekannten Parameter eEvent."

      EndSelect ; #eEvent.

      If Not fDebugViewClear

         ; Die Einrückung der Ausgabe beschränken.
         If (iCounterIndention > #iMaxIndention)
            iCounterIndention = #iMaxIndention
            sProcInOut = #sMaxIndentionSign + sProcInOut
         EndIf

         ; Fortlaufenden Zähler und Debug-Text ausgeben.
         If Len(sModule)
            sProc = sModule + "::"
         EndIf
         If Len(sProcedure)
            sProc + sProcedure
         EndIf
         If iInstance
            sProc + "[" + iInstance + "]"
         EndIf
         If iThread
            sProc + "{" + iThread + "}"
         EndIf
         sProc + ": "
         
         If (Not iThread)
            sDebugOut = Right(#sCounter + Str(miCounterRow), #iCounterWidth) + ":" + sDebugOut +
                        Left(#sIndentionSign, iCounterIndention) + sProcInOut + sInstance + sProc + sText
         Else
            ; Bei Prozeduren die in Threads laufen, keine Einrückung verwenden.
            sDebugOut = Right(#sCounter + Str(miCounterRow), #iCounterWidth) + ":" + sDebugOut +
                        sProcInOut + sInstance + sProc + sText
         EndIf
         miCounterRow + 1
      EndIf
      
      ; Mutex wieder freigeben.
      If (miMutex)
         UnlockMutex(miMutex)
      EndIf

      ; Debugmeldungen an das Betriebssystem ausgeben.
      OutputDebugString_(sDebugOut)

   EndProcedure


EndModule   ; DebugView.
